// Ultra-optimized entry point with code-splitting and prioritized loading
import { startTransition, Suspense, lazy } from "react";
import { createRoot } from "react-dom/client";
import "./index.css";
import { initOfflineFunctionality } from "./utils/offlineUtil";
import { initializeBackgroundSync } from "./utils/registerBackgroundSync";
import { initializeAmazonDeviceOptimizations } from "./lib/amazon-device-optimization";

// Performance metrics for development
const PERFORMANCE_METRICS = process.env.NODE_ENV === 'development';
const startTime = PERFORMANCE_METRICS ? performance.now() : 0;

// Log performance metric for development
const logPerformance = (name: string) => {
  if (PERFORMANCE_METRICS) {
    const time = Math.round(performance.now() - startTime);
    console.log(`⚡️ [Performance] ${name}: ${time}ms`);
  }
};

// Optimized loading states
const updateLoadingMessage = (message: string) => {
  const loaderText = document.querySelector('.loader-text');
  if (loaderText) {
    loaderText.textContent = message;
  }
};

// Remove loading spinner when app ready
const removeLoadingSpinner = () => {
  const spinner = document.querySelector('#root > div.loader-container');
  if (spinner) {
    // Use animation to fade out spinner
    spinner.classList.add('fade-out');
    setTimeout(() => {
      spinner.remove();
    }, 300);
    logPerformance('Initial render complete');
  }
};

// Lazy load App with display name for debugging
const App = lazy(() => {
  updateLoadingMessage('Loading application...');
  return import("./App").then((module) => {
    logPerformance('App module loaded');
    return { default: module.default };
  });
});

// Main initialization function with optimized loading sequence
const initApp = async () => {
  logPerformance('initApp started');
  
  // Initialize offline functionality for PWA
  initOfflineFunctionality();
  
  // Apply Fire OS specific optimizations for Amazon devices
  initializeAmazonDeviceOptimizations();
  
  // Register background sync for offline data persistence
  initializeBackgroundSync().catch(error => {
    console.warn('Background sync initialization failed:', error);
    // App will continue to work even if background sync initialization fails
  });
  
  // Check for cached data to speed up load
  const hasCachedData = localStorage.getItem('app_data_cache') !== null;
  
  // Create root with error boundary
  const rootElement = document.getElementById("root");
  if (rootElement) {
    const root = createRoot(rootElement);
    
    // Set loading text
    updateLoadingMessage(hasCachedData ? 'Loading from cache...' : 'Preparing application...');
    
    // Add CSS fadeout transition
    const style = document.createElement('style');
    style.textContent = '.fade-out { opacity: 0; transition: opacity 0.3s ease-out; }';
    document.head.appendChild(style);
    
    // Optimized with startTransition for improved loading UX
    startTransition(() => {
      root.render(
        <Suspense fallback={null}>
          <App />
        </Suspense>
      );
    });
    
    // We'll keep the spinner until the component actually mounts
    setTimeout(() => {
      // Add a small timeout for the actual render to complete
      // This prevents flashing between loading state and real content
      removeLoadingSpinner();
    }, 300);
  }
};

// Priority loading strategy
const startApp = () => {
  // Check if fonts are already loading
  const fontStatus = (document as any).fonts?.status || 'loading';
  
  if (fontStatus === 'loaded') {
    // Fonts already loaded, initialize immediately
    initApp();
  } else {
    // Wait for fonts or timeout after 800ms
    const timeoutId = setTimeout(() => {
      initApp();
    }, 800);
    
    // Also initialize when fonts load, if they load before timeout
    (document as any).fonts?.ready.then(() => {
      clearTimeout(timeoutId);
      initApp();
    });
  }
};

// Start app based on document readiness state
if (document.readyState === 'loading') {
  // If document still loading, wait for interactive state
  document.addEventListener('DOMContentLoaded', startApp);
} else {
  // If document already interactive/complete, init now
  startApp();
}
